/*
 * MAX : HPmeas.cc, Wed Jan  2 14:44:59 CET 2002 -sg
 *
 * Device driver for HP Digitizing Oscilloscope 54XXXB 
 * 	- measurement only version !
 *
 * M. Dammasch   14.12.1998 error()-handling changed (only 0 returns)
 * 
 * This file is part of Max data acquisition software
 * (c) 1997-98 by C. Otto & M. Dammasch
 *
 * Max is free software; you can redistribute it and/or modify it
 * under the terms of the version 2 of GNU General Public License as
 * published by the Free Software Foundation.
 *
 * Max is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * (see the file COPYING) in this directory; if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

//this header has always to be included !!
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include "list.hh"
#include "sched.hh"
#include "error.hh"

extern "C" {
#include <ugpib.h>
#include <gpiberr.h>
}

//#define HP_MEAS_CMD ":measure:vrms"
//#define HP_MEAS_QUERY ":measure:vrms?"
//#define HP_MEAS_CMD ":measure:vmax"
//#define HP_MEAS_QUERY ":measure:vmax?"
//#define HP_MEAS_CMD ":measure:vmin"
//#define HP_MEAS_QUERY ":measure:vmin?"
#define HP_MEAS_CMD ":measure:vpp"
#define HP_MEAS_QUERY ":measure:vpp?"

#define HP_REFERENCE_LEFT    -1
#define HP_REFERENCE_CENTER  0
#define HP_REFERENCE_RIGHT   1

#define HP_TYPE_REALTIME     0
#define HP_TYPE_NORMAL       0
#define HP_TYPE_SINGLE       0

#define HP_TYPE_REPETITIVE   1
#define HP_TYPE_AVERAGE      1
#define HP_TYPE_TRIGGERED    1

#define HP_TYPE_ENVELOPE     2
#define HP_TYPE_RAWDATA      3

#define HP_FORMAT_COMPRESSED  0
#define HP_FORMAT_BYTE        1
#define HP_FORMAT_WORD        2
#define HP_FORMAT_ASCII       3

#define HP_STRING_MAX  40

//extern int ibsta, ibcnt, iberr;

static int gpib_adress;    // Gpib-adress of the oszi
static char cmd[HP_STRING_MAX+1];
static int TLAM;
static int Channel;        // Channel to digitize
static char errmsg[51];    // local error message string
static char qterrmsg[81];  // compound error message string for qt-error()-widget

static int do_autoscale;   // perform autoscale before measurement flag

static int tb_type;
static double tb_range;
static double tb_delay;
static int tb_delref;

static double yrange;
static double yoffset;

static void cls_gpib(void)
{
  printf("\nclearing HP device...");
  if( ibclr(gpib_adress) & ERR)
    gpiberr("clr err");
}

/*************************************************/
static int HP_wrtcmd ( char *cmdstr )
{
  //ibcnt = 0;
  if ( ibwrt(gpib_adress,cmdstr,strlen(cmdstr)) & ERR)
  {
    gpiberr("wrt err");
    cls_gpib();
    sprintf(errmsg,"\tHP ibwrite err:\nCouldn't send <%s>\n",cmdstr);
    return 0;
  }
  //printf("\nHP-W(%d/%d):<%s>",ibcnt,strlen(cmdstr),cmdstr);
  return 1;
}

static int HP_rdstr ( char *retstr )
{
  ibcnt = 0;
  if ( ibrd(gpib_adress,retstr,HP_STRING_MAX) & ERR)
  {
    gpiberr("read error");
    cls_gpib();
    sprintf(errmsg,"\tHP ibread err:\nReceived <%s>\n",retstr);
    return 0;
  }
  retstr[ibcnt] = '\0';
  //  printf("\nHP-R(%d):<%s>",ibcnt,retstr);
  return 1;
}
///////////////////////////////////////////////////
/*
static int HP_set_timebase ( int type, double range, double delay, int reference )
{
  char cmd[HP_STRING_MAX+1];
  
  if (type == HP_TYPE_REALTIME) sprintf(cmd,":timebase:sample realtime");
  else
  if (type == HP_TYPE_REPETITIVE) sprintf(cmd,":timebase:sample repetitive");
  else return 0;
  if (!HP_wrtcmd(cmd)) return 0;

  if (type == HP_TYPE_REALTIME) sprintf(cmd,":timebase:mode single");
  else
  if (type == HP_TYPE_REPETITIVE) sprintf(cmd,":timebase:mode triggered");
  else return 0;
  if (!HP_wrtcmd(cmd)) return 0;

  sprintf(cmd,":timebase:range %E",range);
  if (!HP_wrtcmd(cmd)) return 0;

  sprintf(cmd,":timebase:delay %E",delay);
  if (!HP_wrtcmd(cmd)) return 0;

  switch (reference)
  {
    case HP_REFERENCE_LEFT :      
      sprintf(cmd,":timebase:reference left");
    break;
    case HP_REFERENCE_CENTER :
      sprintf(cmd,":timebase:reference center");
    break;
    case HP_REFERENCE_RIGHT :
      sprintf(cmd,":timebase:reference right");
    break;
    default :    
      sprintf(errmsg,"\nIllegal reference (%d)\n",reference);
      return 0;
    break;
  }
  if (!HP_wrtcmd(cmd)) return 0;
  
  return 1;
}
*/

static int HP_get_timebase ( int *type, double *range, double *delay, int *reference )
{
  char cmd[HP_STRING_MAX+1], str[HP_STRING_MAX+1];

  sprintf(cmd,":timebase:sample?");
  if (!HP_wrtcmd(cmd)) return 0;
  if (!HP_rdstr(str)) return 0; 
  if (strstr(str,"REAL") != NULL) *type = HP_TYPE_REALTIME;
  else
    if (strstr(str,"REP") != NULL) *type = HP_TYPE_REPETITIVE;
  else
  {
    sprintf(errmsg,"\nIllegal timebase sample type (%s)\n",str);
    return 0;
  }

  sprintf(cmd,":timebase:range?");
  if (!HP_wrtcmd(cmd)) return 0;
  if (!HP_rdstr(str)) return 0;
  *range = atof(str);

  sprintf(cmd,":timebase:delay?");
  if (!HP_wrtcmd(cmd)) return 0;
  if (!HP_rdstr(str)) return 0;
  *delay = atof(str);

  sprintf(cmd,":timebase:reference?");
  if (!HP_wrtcmd(cmd)) return 0;
  if (!HP_rdstr(str)) return 0;

  if (strstr(str,"LEFT") != NULL) *reference = HP_REFERENCE_LEFT;
  else
    if (strstr(str,"CENT") != NULL) *reference = HP_REFERENCE_CENTER;
  else
    if (strstr(str,"RIGH") != NULL) *reference = HP_REFERENCE_RIGHT;
  else
  {
    sprintf(errmsg,"\nIllegal reference (%s)\n",str);
    return 0;
  }
  
  return 1;
}

/*
static int HP_set_channel ( int channel, double range, double offset )
{
  char cmd[HP_STRING_MAX+1];
  
  sprintf(cmd,":channel%d:range %E",channel,range);
  if (!HP_wrtcmd(cmd)) return 0;

  sprintf(cmd,":channel%d:offset %E",channel,offset);
  if (!HP_wrtcmd(cmd)) return 0;

  return 1;
}
*/

static int HP_get_channel ( int channel, double *range, double *offset )
{
  char cmd[HP_STRING_MAX+1], str[HP_STRING_MAX+1];
  
  sprintf(cmd,":channel%d:range?",channel);
  if (!HP_wrtcmd(cmd)) return 0;
  if (!HP_rdstr(str)) return 0;
  *range = atof(str);

  sprintf(cmd,":channel%d:offset?",channel);
  if (!HP_wrtcmd(cmd)) return 0;
  if (!HP_rdstr(str)) return 0;
  *offset = atof(str);

  return 1;
}

/*
static int HP_opc ( void )
{
  char quest[HP_STRING_MAX+1], answ[HP_STRING_MAX+1];

  sprintf(quest,"*OPC?");
  if (!HP_wrtcmd(quest)) return 0;
  if (!HP_rdstr(answ)) return 0;
  if (atoi(answ) == 1) return 1;
  return 0;
}

static int HP_ter ( void )
{
  char quest[HP_STRING_MAX+1], answ[HP_STRING_MAX+1];

  sprintf(quest,":TER?");
  if (!HP_wrtcmd(quest)) return 0;
  if (!HP_rdstr(answ)) return 0;
  if (atoi(answ) == 1) return 1;
  return 0;
}
*/
/*************************************************/

//init function for HP-Scope
//called before meausurement starts
int HPmeas_init(List *device_data, Scheduler *parameters)
{
  //initial HP
  
  printf("\n\n***** Connecting GPIB Device 'HP' *****");
  if((gpib_adress = ibfind("HP")) & ERR){
    gpiberr("ibfind err");
    if (!parameters->error(E_GPIB," HP ibfind err"))
      return 0;   //return FALSE -> measurement stops
  }
  printf("\nHP GPIB-Address is set to #%d",gpib_adress);
    
  printf("\nsend IFC"); //ibsic aus Lib. 
  if( ibsic(gpib_adress) & ERR){
    gpiberr("ibsic err");
    cls_gpib();
    if (!parameters->error(E_GPIB," HP ibsic err"))
      return 0;   //return FALSE -> measurement stops
  }
  printf(" done");

  printf("\nsend REM");
  if( ibsre(gpib_adress,1) & ERR){
    gpiberr("ibsre err");
    cls_gpib();
    if (!parameters->error(E_GPIB," HP ibsre err"))
      return 0;   //return FALSE -> measurement stops
  }
  printf(" done");
  
  cls_gpib();

  printf("\n***** Completed Successfully *****\n");
  
  sprintf(errmsg,"\tNo error(s) occured\n");
  
  device_data->getvar(&TLAM,"all","timeout");
  device_data->getvar(&Channel,"HP_meas","Channel");
  device_data->getvar(&do_autoscale,"HP_meas","do_autoscale");
  device_data->getvar(&tb_type,"HP_meas","tb_type");  
  device_data->getvar(&tb_range,"HP_meas","tb_range");  
  device_data->getvar(&tb_delay,"HP_meas","tb_delay");  
  device_data->getvar(&tb_delref,"HP_meas","tb_delay_ref");  
  device_data->getvar(&yrange,"HP_meas","yrange");  
  device_data->getvar(&yoffset,"HP_meas","yoffset");  

  printf("\ninitializing HP...");
  
  // activate HP
  sprintf(cmd,":run");
  if (ibwrt (gpib_adress, cmd, strlen(cmd)) & ERR) {
    gpiberr("wrt err");
    cls_gpib();
    if (!parameters->error(E_GPIB," HP ibwrite err"))
      return 0;   //return FALSE -> measurement stops
  }
  
  // set communication protocol
  sprintf(cmd,":system:longform off;header off");
  if (ibwrt (gpib_adress, cmd, strlen(cmd) )& ERR ){
    gpiberr("wrt err");
    cls_gpib();
    if (!parameters->error(E_GPIB," HP ibwrite err"))
      return 0;   //return FALSE -> measurement stops
  }  
  
  // choose scope channel for measurement
  sprintf( cmd, ":measure:source channel%d",Channel);
  if (ibwrt (gpib_adress, cmd, strlen(cmd) )& ERR ) {
    gpiberr("wrt err");
    cls_gpib();
    if (!parameters->error(E_IB_WRITE," HP ibwrite err"))
      return 0;   //return FALSE -> measurement stops
  }

  // send 'autoscale' command if flag do_autoscale is set
  if (do_autoscale)
  {
    sprintf( cmd, ":autoscale");
    if (ibwrt (gpib_adress, cmd, strlen(cmd) )& ERR )
    {
      gpiberr("wrt err");
      cls_gpib();
      if (!parameters->error(E_IB_WRITE," HP ibwrite err"))
	return 0;   //return FALSE -> measurement stops
    }
  }
  
  // read timbase settings
  if (HP_get_timebase(&tb_type,&tb_range,&tb_delay,&tb_delref) != 1) {
    sprintf(qterrmsg,"\nHP_GET_TIMEBASE(): Error due to\n%s",errmsg);
    if (!parameters->error(E_GPIB,qterrmsg))
      return 0;
  }  
  
  // read channel settings
  if (HP_get_channel(Channel,&yrange,&yoffset) != 1) {
    sprintf(qterrmsg,"\nHP_GET_CHANNEL(): Error due to\n%s",errmsg);
    if (!parameters->error(E_GPIB,qterrmsg))
      return 0;
  }
  
  printf("\b\b\b done");
  
  return 1;
}


int HPmeas_close(List *device_data, Scheduler *parameters)
{
  printf("closing HP\n");

  //read timebase settings
  if (HP_get_timebase(&tb_type,&tb_range,&tb_delay,&tb_delref) != 1)
  {
    sprintf(qterrmsg,"\nHP_GET_TIMEBASE(): Error due to\n%s",errmsg);
    if (!parameters->error(E_GPIB,qterrmsg))
      return 0;
  }  

  // read channel settings
  if (HP_get_channel(Channel,&yrange,&yoffset) != 1)
  {
    sprintf(qterrmsg,"\nHP_GET_CHANNEL(): Error due to\n%s",errmsg);
    if (!parameters->error(E_GPIB,qterrmsg))
      return 0;
  }   

  // end up measurement: reset scope
  sprintf( cmd, "*cls"); //Clear Memory
  if (ibwrt (gpib_adress, cmd, strlen(cmd) )& ERR ){
    gpiberr("wrt err");
    cls_gpib();
    if (!parameters->error(E_IB_WRITE,"HP ibwrite err"))
      return 0;   //return FALSE -> measurement stops
  }  

  // set scope to local
  sprintf( cmd, ":system:key 39");  // Keycode #39 - LOCAL
  if (ibwrt (gpib_adress, cmd, strlen(cmd) )& ERR ){
    gpiberr("wrt err");
    cls_gpib();
    if (!parameters->error(E_IB_WRITE,"HP ibwrite err"))
      return 0;   //return FALSE -> measurement stops
  }   

  device_data->putvar(Channel,"HP_meas","Channel");
  device_data->putvar(do_autoscale,"HP_meas","do_autoscale");
  device_data->putvar(tb_type,"HP_meas","tb_type");  
  device_data->putvar(tb_range,"HP_meas","tb_range");  
  device_data->putvar(tb_delay,"HP_meas","tb_delay");  
  device_data->putvar(tb_delref,"HP_meas","tb_delay_ref");  
  device_data->putvar(yrange,"HP_meas","yrange");  
  device_data->putvar(yoffset,"HP_meas","yoffset");  
  
  return 1; //function has to return a value -> 1 otherwise measurement stops
}


//pre-trigger device function for HP
int HPmeas_pre(List *device_data, Scheduler *parameters) 
{  
  // send measure command if HPmeas device enabled AND very 1st channel
  if (device_data->enabled("HP_meas","HPmeas") && parameters->act_channel() == 1) {
    sprintf( cmd, HP_MEAS_CMD);
    if (ibwrt (gpib_adress, cmd, strlen(cmd) )& ERR )
      {
	gpiberr("wrt err");
	cls_gpib();
	if (!parameters->error(E_IB_WRITE," HP ibwrite err"))
	  return 0;   //return FALSE -> measurement stops
      }
  }

  return 1;
}


//device function for HP
int HPmeas(List *device_data, Scheduler *parameters) 
{
  
  char   datastr[HP_STRING_MAX+1];
  double data;

  // if not enabled quit immediately
  if (!device_data->enabled("HP_meas","HPmeas")) return 1;
 
  // ask for measured value
  sprintf( cmd, HP_MEAS_QUERY);
  if (!HP_wrtcmd(cmd))
  {
    sprintf(qterrmsg,"\nHP_MEAS(): Error due to\n%s",errmsg);
    if (!parameters->error(E_GPIB,qterrmsg))
      return 0;
  }  
  
  if (!HP_rdstr(datastr))
  {
    sprintf(qterrmsg,"\nHP_MEAS(): Error due to\n%s",errmsg);
    if (!parameters->error(E_GPIB,qterrmsg))
      return 0;
  }  
  data = atof(datastr);

  if (device_data->enabled("HP_meas","HPmeas")) {
    parameters->store(data);	// save the measurement result
    printf("\nHP_meas: data stored (%G)",data);
  }

  /*  
  // end up read out process: reset scope
  sprintf( cmd, "*cls"); //Clear Memory
  if (ibwrt (gpib_adress, cmd, strlen(cmd) )& ERR ){
    gpiberr("wrt err");
    cls_gpib();
    if (!parameters->error(E_IB_WRITE,"HP ibwrite err"))
      return 0;   //return FALSE -> measurement stops
  }
  */

  //printf("\nHP_meas: measurement done (%d)",parameters->act_channel());

  return 1; 
}








