/*
 * MAX : HPro.cc, Wed Jan  2 14:44:59 CET 2002 -sg
 *
 * Device driver for HP Digitizing Oscilloscope 54XXXB 
 * 	- readonly only version !
 *
 * This file is part of Max data acquisition software
 * (c) 1997-98 by M. Dammasch
 *
 * Max is free software; you can redistribute it and/or modify it
 * under the terms of the version 2 of GNU General Public License as
 * published by the Free Software Foundation.
 *
 * Max is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * (see the file COPYING) in this directory; if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

//this header has always to be included !!
#include "list.h"
#include "sched.h"
#include "error.h"
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include "gpiberr.h"
#include <ib.h>

#define HP_REFERENCE_LEFT    -1
#define HP_REFERENCE_CENTER  0
#define HP_REFERENCE_RIGHT   1

#define HP_TYPE_REALTIME     0
#define HP_TYPE_NORMAL       0
#define HP_TYPE_SINGLE       0

#define HP_TYPE_REPETITIVE   1
#define HP_TYPE_AVERAGE      1
#define HP_TYPE_TRIGGERED    1

#define HP_TYPE_ENVELOPE     2
#define HP_TYPE_RAWDATA      3

#define HP_FORMAT_COMPRESSED  0
#define HP_FORMAT_BYTE        1
#define HP_FORMAT_WORD        2
#define HP_FORMAT_ASCII       3

#define HP_STRING_MAX  40

extern int ibsta, ibcnt, iberr;

static int gpib_adress;    // Gpib-adress of the oszi
static char cmd[HP_STRING_MAX+1];
static int TLAM;
static int Channel;        // Channel to digitize
static int result;
static char errmsg[51];    // local error message string
static char qterrmsg[81];  // compound error message string for qt-error()-widget

static int tb_type;
static double tb_range;
static double tb_delay;
static int tb_delref;

static double yrange;
static double yoffset;

static int aq_type;
static int aq_format;
static int aq_counts;
static int aq_points;
static int aq_complete;
static int aq_del;

static void cls_gpib(void)
{
  printf("\nclearing HP device...");
  if( ibclr(gpib_adress) & ERR)
    gpiberr("clr err");
}

/*************************************************/
static int HP_wrtcmd ( char *cmdstr )
{
  //ibcnt = 0;
  if ( ibwrt(gpib_adress,cmdstr,strlen(cmdstr)) & ERR)
  {
    gpiberr("wrt err");
    cls_gpib();
    sprintf(errmsg,"\tHP ibwrite err:\nCouldn't send <%s>\n",cmdstr);
    return 0;
  }
  //printf("\nHP-W(%d/%d):<%s>",ibcnt,strlen(cmdstr),cmdstr);
  return 1;
}

static int HP_rdstr ( char *retstr )
{
  ibcnt = 0;
  if ( ibrd(gpib_adress,retstr,HP_STRING_MAX) & ERR)
  {
    gpiberr("read error");
    cls_gpib();
    sprintf(errmsg,"\tHP ibread err:\nReceived <%s>\n",retstr);
    return 0;
  }
  retstr[ibcnt] = '\0';
  //  printf("\nHP-R(%d):<%s>",ibcnt,retstr);
  return 1;
}
///////////////////////////////////////////////////
static int HP_set_timebase ( int type, double range, double delay, int reference )
{
  char cmd[HP_STRING_MAX+1];
  
  if (type == HP_TYPE_REALTIME) sprintf(cmd,":timebase:sample realtime");
  else
  if (type == HP_TYPE_REPETITIVE) sprintf(cmd,":timebase:sample repetitive");
  else return 0;
  if (!HP_wrtcmd(cmd)) return 0;

  if (type == HP_TYPE_REALTIME) sprintf(cmd,":timebase:mode single");
  else
  if (type == HP_TYPE_REPETITIVE) sprintf(cmd,":timebase:mode triggered");
  else return 0;
  if (!HP_wrtcmd(cmd)) return 0;

  sprintf(cmd,":timebase:range %E",range);
  if (!HP_wrtcmd(cmd)) return 0;

  sprintf(cmd,":timebase:delay %E",delay);
  if (!HP_wrtcmd(cmd)) return 0;

  switch (reference)
  {
    case HP_REFERENCE_LEFT :      
      sprintf(cmd,":timebase:reference left");
    break;
    case HP_REFERENCE_CENTER :
      sprintf(cmd,":timebase:reference center");
    break;
    case HP_REFERENCE_RIGHT :
      sprintf(cmd,":timebase:reference right");
    break;
    default :    
      sprintf(errmsg,"\nIllegal reference (%d)\n",reference);
      return 0;
    break;
  }
  if (!HP_wrtcmd(cmd)) return 0;
  
  return 1;
}

static int HP_get_timebase ( int *type, double *range, double *delay, int *reference )
{
  char cmd[HP_STRING_MAX+1], str[HP_STRING_MAX+1];

  sprintf(cmd,":timebase:sample?");
  if (!HP_wrtcmd(cmd)) return 0;
  if (!HP_rdstr(str)) return 0; 
  if (strstr(str,"REAL") != NULL) *type = HP_TYPE_REALTIME;
  else
    if (strstr(str,"REP") != NULL) *type = HP_TYPE_REPETITIVE;
  else
  {
    sprintf(errmsg,"\nIllegal timebase sample type (%s)\n",str);
    return 0;
  }

  sprintf(cmd,":timebase:range?");
  if (!HP_wrtcmd(cmd)) return 0;
  if (!HP_rdstr(str)) return 0;
  *range = atof(str);

  sprintf(cmd,":timebase:delay?");
  if (!HP_wrtcmd(cmd)) return 0;
  if (!HP_rdstr(str)) return 0;
  *delay = atof(str);

  sprintf(cmd,":timebase:reference?");
  if (!HP_wrtcmd(cmd)) return 0;
  if (!HP_rdstr(str)) return 0;

  if (strstr(str,"LEFT") != NULL) *reference = HP_REFERENCE_LEFT;
  else
    if (strstr(str,"CENT") != NULL) *reference = HP_REFERENCE_CENTER;
  else
    if (strstr(str,"RIGH") != NULL) *reference = HP_REFERENCE_RIGHT;
  else
  {
    sprintf(errmsg,"\nIllegal reference (%s)\n",str);
    return 0;
  }
  
  return 1;
}

static int HP_set_channel ( int channel, double range, double offset )
{
  char cmd[HP_STRING_MAX+1];
  
  sprintf(cmd,":channel%d:range %E",channel,range);
  if (!HP_wrtcmd(cmd)) return 0;

  sprintf(cmd,":channel%d:offset %E",channel,offset);
  if (!HP_wrtcmd(cmd)) return 0;

  return 1;
}

static int HP_get_channel ( int channel, double *range, double *offset )
{
  char cmd[HP_STRING_MAX+1], str[HP_STRING_MAX+1];
  
  sprintf(cmd,":channel%d:range?",channel);
  if (!HP_wrtcmd(cmd)) return 0;
  if (!HP_rdstr(str)) return 0;
  *range = atof(str);

  sprintf(cmd,":channel%d:offset?",channel);
  if (!HP_wrtcmd(cmd)) return 0;
  if (!HP_rdstr(str)) return 0;
  *offset = atof(str);

  return 1;
}

static int HP_set_acquire ( int type, int format, int points, int counts, int complete )
{
  char cmd[HP_STRING_MAX+1];
  
  /*  
  switch (type)
  {
    case HP_TYPE_NORMAL :
      sprintf(cmd,":timebase:sample realtime");
      if (!HP_wrtcmd(cmd)) return 0;
      sprintf(cmd,":timebase:mode single");
      if (!HP_wrtcmd(cmd)) return 0;
      sprintf(cmd,":acquire:type normal");
      if (!HP_wrtcmd(cmd)) return 0;
    break;
    case HP_TYPE_AVERAGE :
      sprintf(cmd,":timebase:sample repetitive");
      if (!HP_wrtcmd(cmd)) return 0;
      sprintf(cmd,":timebase:mode triggered");
      if (!HP_wrtcmd(cmd)) return 0;
      sprintf(cmd,":acquire:type average");
      if (!HP_wrtcmd(cmd)) return 0;
    break;
    case HP_TYPE_ENVELOPE :
      sprintf(errmsg,"\nacquire type 'envelope' not supported yet");
      return 0;
    break;
    case HP_TYPE_RAWDATA :
      sprintf(errmsg,"\nacquire type 'rawdata' not supported yet");
      return 0;
    break;
    default :
      sprintf(errmsg,"\nIllegal type (%d)\n",type);
      return 0;
    break;
  }
  */
 
  switch (format)
  {
    case HP_FORMAT_COMPRESSED :
      sprintf(cmd,":waveform:format comp");
      if (!HP_wrtcmd(cmd)) return 0;
    break;
    case HP_FORMAT_BYTE :
      sprintf(cmd,":waveform:format byte");
      if (!HP_wrtcmd(cmd)) return 0;
    break;
    case HP_FORMAT_WORD :
      sprintf(cmd,":waveform:format word");
      if (!HP_wrtcmd(cmd)) return 0;
    break;
    case HP_FORMAT_ASCII :
      sprintf(cmd,":waveform:format ascii");
      if (!HP_wrtcmd(cmd)) return 0;
    break;
    default :
      sprintf(errmsg,"\nIllegal format (%d)\n",format);
      return 0;
    break; 
  }

  /*
  sprintf(cmd,":acquire:points %d",((points<1024)?(500):(8000)));
  if (!HP_wrtcmd(cmd)) return 0;
  sprintf(cmd,":acquire:count %d",counts);
  if (!HP_wrtcmd(cmd)) return 0;
  sprintf(cmd,":acquire:complete %d",complete);
  if (!HP_wrtcmd(cmd)) return 0;
  */

  return 1;
}

static int HP_get_acquire ( int *type, int *format, int *points, int *counts, int *complete )
{
  char cmd[HP_STRING_MAX+1], str[HP_STRING_MAX+1];
  
  sprintf(cmd,":acquire:type?");
  if (!HP_wrtcmd(cmd)) return 0;
  if (!HP_rdstr(str)) return 0;
  if (strstr(str,"NORM") != NULL) *type = HP_TYPE_NORMAL;
  else
    if (strstr(str,"AVER") != NULL) *type = HP_TYPE_AVERAGE;
  else
    if (strstr(str,"ENV") != NULL) *type = HP_TYPE_ENVELOPE;
  else
    if (strstr(str,"RAWD") != NULL) *type = HP_TYPE_RAWDATA;
  else
  {
    sprintf(errmsg,"\nIllegal type (%s)\n",str);
    return 0;
  }

  sprintf(cmd,":waveform:format?");
  if (!HP_wrtcmd(cmd)) return 0;
  if (!HP_rdstr(str)) return 0;
  if (strstr(str,"COMP") != NULL) *format = HP_FORMAT_COMPRESSED;
  else
    if (strstr(str,"BYTE") != NULL) *format = HP_FORMAT_BYTE;
  else
    if (strstr(str,"WORD") != NULL) *format = HP_FORMAT_WORD;
  else
    if (strstr(str,"ASC") != NULL) *format = HP_FORMAT_ASCII;
  else
  {
    sprintf(errmsg,"\nIllegal format (%s)\n",str);
    return 0;
  }

  sprintf(cmd,":acquire:points?");
  if (!HP_wrtcmd(cmd)) return 0;
  if (!HP_rdstr(str)) return 0;
  *points = atoi(str);

  sprintf(cmd,":acquire:count?");
  if (!HP_wrtcmd(cmd)) return 0;
  if (!HP_rdstr(str)) return 0;
  *counts = atoi(str);

  sprintf(cmd,":acquire:complete?");
  if (!HP_wrtcmd(cmd)) return 0;
  if (!HP_rdstr(str)) return 0;
  *complete = atoi(str);

  return 1;
}

static int HP_opc ( void )
{
  char quest[HP_STRING_MAX+1], answ[HP_STRING_MAX+1];

  sprintf(quest,"*OPC?");
  if (!HP_wrtcmd(quest)) return 0;
  if (!HP_rdstr(answ)) return 0;
  if (atoi(answ) == 1) return 1;
  return 0;
  
}

static int HP_ter ( void )
{
  char quest[HP_STRING_MAX+1], answ[HP_STRING_MAX+1];

  sprintf(quest,":TER?");
  if (!HP_wrtcmd(quest)) return 0;
  if (!HP_rdstr(answ)) return 0;
  if (atoi(answ) == 1) return 1;
  return 0;
  
}
/*************************************************/

//init function for HP-Scope
//called before meausurement starts
int HPro_init(List &device_data, Scheduler *parameters)
{
  //initial HP
  
  printf("\n\n***** Connecting GPIB Device 'HP' *****");
  if((gpib_adress = ibfind("HP")) & ERR){
    gpiberr("ibfind err");
    result=error(E_GPIB," HP ibfind err");
    return result;   //return FALSE -> measurement stops
  }
  printf("\nHP GPIB-Address is set to #%d",gpib_adress);
    
  printf("\nsend IFC"); //ibsic aus Lib. 
  if( ibsic(gpib_adress) & ERR){
    gpiberr("ibsic err");
    cls_gpib();
    result=error(E_GPIB," HP ibsic err");
    return result;   //return FALSE -> measurement stops
  }
  printf(" done");

  printf("\nsend REM");
  if( ibsre(gpib_adress,1) & ERR){
    gpiberr("ibsre err");
    cls_gpib();
    result=error(E_GPIB," HP ibsre err");
    return result;   //return FALSE -> measurement stops
  }
  printf(" done");
  
  cls_gpib();

  printf("\n***** Completed Successfully *****\n");
  
  sprintf(errmsg,"\tNo error(s) occured\n");
  
  device_data.getvar(&TLAM,"all","timeout");
  device_data.getvar(&Channel,"HP_ro","Channel");
  device_data.getvar(&aq_type,"HP_ro","acq_type");  
  device_data.getvar(&aq_format,"HP_ro","acq_format");
  device_data.getvar(&aq_counts,"HP_ro","acq_count");  
  device_data.getvar(&aq_points,"HP_ro","acq_point");  
  device_data.getvar(&aq_complete,"HP_ro","acq_complete");  
  device_data.getvar(&aq_del,"HP_ro","acq_delay");  
  device_data.getvar(&tb_type,"HP_ro","tb_type");  
  device_data.getvar(&tb_range,"HP_ro","tb_range");  
  device_data.getvar(&tb_delay,"HP_ro","tb_delay");  
  device_data.getvar(&tb_delref,"HP_ro","tb_delay_ref");  
  device_data.getvar(&yrange,"HP_ro","yrange");  
  device_data.getvar(&yoffset,"HP_ro","yoffset");  
  
  printf("\ninitializing HP...");

  sprintf(cmd,":system:longform off;header off");
  if (ibwrt (gpib_adress, cmd, strlen(cmd) )& ERR ){
    gpiberr("wrt err");
    cls_gpib();
    result=error(E_GPIB," HP ibwrite err");
    return result;   //return FALSE -> measurement stops
  }

  if (HP_set_acquire(aq_type,aq_format,aq_points,aq_counts,aq_complete) != 1)
  {
    sprintf(qterrmsg,"\nHP_SET_ACQUIRE(): Error due to\n%s",errmsg);
    result = error(E_GPIB,qterrmsg);
    return result;
  } 

  if (HP_get_timebase(&tb_type,&tb_range,&tb_delay,&tb_delref) != 1)
  {
    sprintf(qterrmsg,"\nHP_GET_TIMEBASE(): Error due to\n%s",errmsg);
    result = error(E_GPIB,qterrmsg);
    return result;
  }  
  if (HP_get_channel(Channel,&yrange,&yoffset) != 1)
  {
    sprintf(qterrmsg,"\nHP_GET_CHANNEL(): Error due to\n%s",errmsg);
    result = error(E_GPIB,qterrmsg);
    return result;
  }   
  if (HP_get_acquire(&aq_type,&aq_format,&aq_points,&aq_counts,&aq_complete) != 1)
  {
    sprintf(qterrmsg,"\nHP_GET_ACQUIRE(): Error due to\n%s",errmsg);
    result = error(E_GPIB,qterrmsg);
    return result;
  }
  
  sprintf( cmd, ":waveform:points?"); // number of transmitted bytes (comp format only!)
  if (!HP_wrtcmd(cmd)) return 0;
  if (!HP_rdstr(cmd)) return 0;
  if (aq_points != atoi(cmd))
  {
    sprintf(qterrmsg,"HP_INIT(): Different values for AQ_POINTS (%d) and WAV_POINTS (%d)",aq_points,atoi(cmd));
    result = error(E_GPIB,qterrmsg);
    return result;
  }  
  
  printf("\b\b\b done");
  return 1;
}


int HPro_close(List &device_data, Scheduler *parameters)
{
  printf("closing HP\n");

  if (HP_get_timebase(&tb_type,&tb_range,&tb_delay,&tb_delref) != 1)
  {
    sprintf(qterrmsg,"\nHP_GET_TIMEBASE(): Error due to\n%s",errmsg);
    result = error(E_GPIB,qterrmsg);
    return result;
  }  
  if (HP_get_channel(Channel,&yrange,&yoffset) != 1)
  {
    sprintf(qterrmsg,"\nHP_GET_CHANNEL(): Error due to\n%s",errmsg);
    result = error(E_GPIB,qterrmsg);
    return result;
  }   
  if (HP_get_acquire(&aq_type,&aq_format,&aq_points,&aq_counts,&aq_complete) != 1)
  {
    sprintf(qterrmsg,"\nHP_GET_ACQUIRE(): Error due to\n%s",errmsg);
    result = error(E_GPIB,qterrmsg);
    return result;
  }  
  sprintf( cmd, ":system:key 39");  // Keycode #39 - LOCAL
  if (ibwrt (gpib_adress, cmd, strlen(cmd) )& ERR ){
    gpiberr("wrt err");
    cls_gpib();
    result=error(E_IB_WRITE,"HP ibwrite err");
    return result;   //return FALSE -> measurement stops
  }   

  device_data.putvar(Channel,"HP_ro","Channel");
  device_data.putvar(tb_type,"HP_ro","tb_type");  
  device_data.putvar(tb_range,"HP_ro","tb_range");  
  device_data.putvar(tb_delay,"HP_ro","tb_delay");  
  device_data.putvar(tb_delref,"HP_ro","tb_delay_ref");  
  device_data.putvar(yrange,"HP_ro","yrange");  
  device_data.putvar(yoffset,"HP_ro","yoffset");  
  device_data.putvar(aq_type,"HP_ro","acq_type");  
  device_data.putvar(aq_format,"HP_ro","acq_format");  
  device_data.putvar(aq_counts,"HP_ro","acq_count");  
  device_data.putvar(aq_points,"HP_ro","acq_point");  
  device_data.putvar(aq_complete,"HP_ro","acq_complete");  

  return 1; //function has to reutrn a vlue -> >1 otherwise measurement stops
}


//pre-trigger device function for HP
int HPro_pre(List &device_data, Scheduler *parameters) 
{  
  /* HP  aktivieren */

  if (parameters->act_channel() == 1)
  {
    sprintf( cmd, ":system:longform off;header off");
    if (ibwrt (gpib_adress, cmd, strlen(cmd) )& ERR )
    {
      gpiberr("wrt err");
      cls_gpib();
      result=error(E_IB_WRITE,"HP ibwrite err");
      return result;   //return FALSE -> measurement stops
    } 

    sprintf( cmd, ":waveform:source channel%d",Channel);
    if (ibwrt (gpib_adress, cmd, strlen(cmd) )& ERR )
    {
      gpiberr("wrt err");
      cls_gpib();
      result=error(E_IB_WRITE," HP ibwrite err");
      return result;   //return FALSE -> measurement stops
    }

    if (HP_set_acquire(aq_type,aq_format,aq_points,aq_counts,aq_complete) != 1)
    {
      sprintf(qterrmsg,"\nHP_SET_ACQUIRE(): Error due to\n%s",errmsg);
      result = error(E_GPIB,qterrmsg);
      return result;
    } 
  }
    
  printf("\ncapturing data from channel %d, please wait...",Channel);
  sprintf( cmd, ":run");
  if (ibwrt (gpib_adress, cmd, strlen(cmd) )& ERR )
  {
    gpiberr("wrt err");
    cls_gpib();
    result=error(E_IB_WRITE," HP ibwrite err");
    return result;   //return FALSE -> measurement stops
  }
  
  return 1;
}


//device function for HP
int HPro(List &device_data, Scheduler *parameters) 
{
  
  unsigned char *waveform, *tp, token[6];
  char           datahead[11], eostr[2];
  int           *wave, wpmax, datamax, i;

  // wait until data is available
  sleep(aq_del);
  printf(" done");
    
  sprintf( cmd, ":waveform:data?");       //ask for data
  if (ibwrt (gpib_adress, cmd, strlen(cmd) )& ERR ){
    gpiberr("wrt err");
    cls_gpib();
    result=error(E_IB_WRITE,"HP ibwrite err");
    return result;   //return FALSE -> measurement stops
  }

  if (aq_format < HP_FORMAT_ASCII)  // only BYTE, COMP & WORD start with header
  {
    for (i = 0; i < 11; datahead[i++] = '\0');
    if( ibrd (gpib_adress, datahead, 10)  & ERR)	// read data header info
    {
      gpiberr("read error");
      cls_gpib();
      result=error(E_IB_READ,"HP ibread err for 'datahead'");
      return result;   //return FALSE -> measurement stops
    }
    datamax = atoi(datahead+2);  // number of waveform data bytes 
  }
  else datamax = 6*aq_points;  // ASCII: 16bit data words ("0"..."32767") sep. by ','

  if ((waveform = (unsigned char *)calloc(datamax,sizeof(unsigned char))) == NULL)
  {
    sprintf(qterrmsg,"\nCouldn't allocate vector waveform (%d uchars)\n",datamax);
    result = error(E_SYS_MEM,qterrmsg);
    return result;
  }

  switch (aq_format)
  {
    case HP_FORMAT_WORD :
      wpmax = datamax/2;
    break;
    case HP_FORMAT_BYTE :
    case HP_FORMAT_COMPRESSED :
      wpmax = datamax;
    break;
    case HP_FORMAT_ASCII :
    default :
      wpmax = aq_points;
    break;
  }

  if ((wave = (int *)calloc(wpmax,sizeof(int))) == NULL)
  {
    free(waveform);
    sprintf(qterrmsg,"\nCouldn't allocate vector wave (%d ints)\n",
            wpmax);
    result = error(E_SYS_MEM,qterrmsg);
    return result;
  }

  printf("\nreading HP scope data (%d uchars for %d items)...",datamax,wpmax);  
  if( ibrd (gpib_adress, waveform, datamax)  & ERR)
  {
    gpiberr("read error");
    cls_gpib();
    result=error(E_IB_READ,"HP ibread err");
    return result;   //return FALSE -> measurement stops
  }
  
  // convert received GPIB-string 'waveform' into array of ints 'wave'
  switch (aq_format)
  {
    case HP_FORMAT_ASCII :
      i = 0; 
      while ((tp = strtok(((i==0)?(waveform):(NULL)),",")) != NULL)
      {
        strcpy(token,tp);
        wave[i++] = atoi(token);
      }
    break;
    case HP_FORMAT_WORD :
      for(i = 0; i < wpmax; i++)
        wave[i] = ((int)waveform[2*i] << 8) + (int)waveform[2*i+1];
    break;
    case HP_FORMAT_BYTE :
    case HP_FORMAT_COMPRESSED:
      for(i = 0; i < wpmax; wave[i] = (int)waveform[i++]);
    break;
  }

  eostr[0] = eostr[1] = '\0';  
  if( ibrd (gpib_adress, eostr, 1)  & ERR)	// read data <terminator>
  {
    gpiberr("read error");
    cls_gpib();
    result=error(E_IB_READ,"HP ibread err for 'eostr'");
    return result;   //return FALSE -> measurement stops
  }
  
  parameters->store(wave,aq_points);	// save the wave data array
  
  // end up read out process: reset scope
  sprintf( cmd, "*cls"); //Clear Memory
  if (ibwrt (gpib_adress, cmd, strlen(cmd) )& ERR ){
     gpiberr("wrt err");
    cls_gpib();
    result=error(E_IB_WRITE,"HP ibwrite err");
    return result;   //return FALSE -> measurement stops
  }
  printf("\b\b\b end readout HP\n");
  cls_gpib();

  free(waveform);
  free(wave);

  return 1; 
}








