/*
 * MAX : data.cc, Sam Dez 29 19:29:02 CET 2001
 *
 * This file is part of Max data acquisition software
 * Copyright (C) 1998 Christian Rosen
 *
 * Max is free software; you can redistribute it and/or modify it
 * under the terms of the version 2 of GNU General Public License as
 * published by the Free Software Foundation.
 *
 * Max is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * (see the file COPYING) in this directory; if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */

//data storage and handling classes

#include "data.h"
#include "packet.h"
#include "definitions.h"

#include <assert.h>
#include <stdio.h>
//#include <stdlib.h>
#include <math.h>

// return 1 if list is empty
int PacketList::empty() {  return L == 0 ? 1 : 0 ; }
int PacketList::length() {
  PacketElement *Pe= L;
  int  x  = 0;
  while (Pe) {
    Pe=Pe-> next;
    x++;
  }
  return x;
}

//returns pointer to last element in the list
PacketElement *PacketList::last() {
  if (L == 0) {
    printf("last : list is empty \n");
    return L;//List is empty
  }
  PacketElement  *prev,*act;
  prev = act = L;
  while ( act ){
    prev = act;
    act = act -> next;
  }
  L2 = prev;
  return prev;
}

//return pointer to beginning of list
//and set secondary pointer there
PacketElement *PacketList::first() {
  L2 = L;
  return L;
}
PacketElement *PacketList::next() {
  L2 = L2 -> next;
  return L2;
}

//return the data structure containing the packet
dataPacket *PacketList::packet() {
  return L2 -> dat_packet;
}

//erase list
void PacketList::remove() {
  PacketElement *el = L;
  while (el) {
    PacketElement *tmp = el;
    el=el->next;
    delete tmp;
  }
  L  = 0; //reset both pointers
  L2 = L;
}

//append one data set to the list and convert it to a
//datapacket of unsigned char
void PacketList::append(int data,int channel,int prio,int index) {
  //  printf("int appending packet prio %d, chan  %d, index %d \n",prio,channel,index);
  int i;
  //  P = new dataPacket;
  memset(P,0,FULL_P_SIZE);
  P->type      = 'a'; //type is int
  memcpy(P -> priority,&prio,sizeof(int) );
  memcpy(P -> var_index,&index,sizeof(int) );
  memcpy(P-> channel,&channel,sizeof(int) );
  //data length contains the length of the buffer
  //here just one  integer the  in buffer
  int d = sizeof(int);
  memcpy( P->data_length,&d,sizeof(int));
  memcpy( P->t_length   ,&d,sizeof(int)); //total length is one int
  memcpy( P -> buffer,/*(unsigned char *)*/ &data,sizeof(int) );
  for (i = sizeof(int);i<DATA_PACKET_SIZE;i++) { //fill up the rest with space
      P->buffer[i]=0;
  }
  PacketElement *l = new PacketElement(P); //create new packet
  if ( L == 0 ) {      //list is empty -> insert packet at beginning
    L = l;
    return;
  }
  PacketElement *end = last();
  end -> next = l;      //append packet at the end
  //  delete P;
}
void PacketList::append(double data,int channel,int prio,int index) {
  //P = new dataPacket;
  memset(P,0,FULL_P_SIZE);
  P->type      = 'b'; //type is double
  memcpy(P -> priority,&prio,  sizeof(int) );
  memcpy(P -> var_index,&index,sizeof(int) );
  memcpy(P -> channel,&channel,sizeof(int) );
  int d = sizeof(double);
  memcpy( P->data_length,&d,sizeof(int));
  memcpy( P->t_length   ,&d,sizeof(int)); //total length is one int
  memcpy( P -> buffer,/*(unsigned char *)*/ &data,sizeof(double) );
  PacketElement *l = new PacketElement(P); //create new packet
  if ( L == 0 ) {      //list is empty -> insert packet at beginning
    L = l;
    return;
  }
  PacketElement *end = last();
  end -> next = l;      //append packet at the end
  //delete P;	
}

//append an array of double to the list
//data is split in several packets if array is longer
//than DATA_PACKET_SIZE (-> definitions.hpp) / sizeof(double)
void PacketList::append(double *data,int length,int channel,int prio,int index) {
  int number;
  int x,pc;
  int n_elements = DATA_PACKET_SIZE/sizeof(double);
  int pos        = 0; //pointer to position in  dpacket->buffer[]
  int pos2       = 0; //pointer to position in data array
  int n_packets  = 1; //number packets to send this array
  double no_packets;
  no_packets = ceil((double) length/ (double)DATA_PACKET_SIZE*sizeof(double) );
  n_packets = (int) no_packets;
  //loop over number of packets to be made for this array
  for (pc = 0; pc < n_packets; pc++) {
    // P = new dataPacket;
    memset(P,0,FULL_P_SIZE);
    P->type      = 'b'; //type is double
    memcpy(P -> priority,&prio,sizeof(int) );
    memcpy(P -> var_index,&index,sizeof(int) );
    memcpy(P -> channel,&channel,sizeof(int) );
    memcpy( P-> t_length,&length,sizeof(int) );
    pos =0; //position in buffer
    x=0;     //now copy array in buffer
    if (length*sizeof(double)-pc*DATA_PACKET_SIZE > DATA_PACKET_SIZE)
      number = DATA_PACKET_SIZE;
    else
      number = (length*sizeof(double)-pc*DATA_PACKET_SIZE);
    memcpy(P->data_length,&number,sizeof(double));
    memcpy(P->buffer,&data[pos2],number);
    pos2 += n_elements;
    PacketElement *l = new PacketElement(P); //create new packet
    if ( L == 0 ) {      //list is empty -> insert packet at beginning
      L = l;
      return;
    }
    PacketElement *end = last();
    end -> next = l;      //append packet at the end
  }
}

//append an array of int to the list
//data is split into several packets if array is longer
//than DATA_PACKET_SIZE (-> definitions.hpp) / sizeof(int)
void PacketList::append(int *data,int length,int channel,int prio,int index) {
  int number;
  int x,pc;
  int n_elements = DATA_PACKET_SIZE/sizeof(int);
  int pos        = 0; //pointer to position in  dpacket->buffer[]
  int pos2       = 0; //pointer to position in data array
  int n_packets  = 1; //number packets to send this array
  double no_packets;
  no_packets = ceil((double) length/ (double)DATA_PACKET_SIZE*sizeof(int) );
  n_packets = (int) no_packets;

  for (pc = 0; pc < n_packets; pc++) {

    memset(P,0,FULL_P_SIZE);
    P->type      = 'a'; //type is int
    memcpy(P -> priority ,&prio   ,sizeof(int) );
    memcpy(P -> var_index,&index  ,sizeof(int) );
    memcpy(P -> channel  ,&channel,sizeof(int) );
    memcpy(P->t_length,&length,sizeof(int) );   //copy total length of size dataset
    //now copy array in buffer
    pos =0; //position in buffer
    x=0;
    if (length*sizeof(int)-pc*DATA_PACKET_SIZE > DATA_PACKET_SIZE)
      number = DATA_PACKET_SIZE;
    else
      number = (length*sizeof(int)-pc*DATA_PACKET_SIZE);
    memcpy(P->data_length,&number,sizeof(int));
    memcpy(P->buffer,&data[pos2],number);
    pos2 += n_elements;
    PacketElement *l = new PacketElement(P); //create new packet
    assert( l != 0);

    if ( L == 0 ) {      //list is empty -> insert packet at beginning
      L = l;
    }
    else {
      PacketElement *end = last();
      end -> next = l;      //append packet at the end
    }
  }
  //delete P;
}
