/*
 * MAX : gpiberr.cc, 16.08.2002 -sg
 * 
 * Errorhandling functions for Linux Gpib
 * 	ported to MAX 			10.07.97 C. Otto
 * 	ported to GPIBTEST 		13.08.98 M. Dammasch
 * 	better programming style	16.08.02 S. Gerlach
 *
 * This file is part of Max data acquisition software
 * Copyright (C) 1997,98 Christian Rosen
 *
 * Max is free software; you can redistribute it and/or modify it
 * under the terms of the version 2 of GNU General Public License as
 * published by the Free Software Foundation.
 *
 * Max is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * (see the file COPYING) in this directory; if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdarg.h>

#ifdef _NI_GPIB_
#include <ugpib.h>
#endif

#ifdef _LINUX_GPIB_
#include <ib.h>
#endif

#include "gpiberr.h"


//  This is a simple error handling function
void  gpib_error ( char *formatstr, ... ) {
	char     msg[256];
  	va_list  args;

  	va_start(args,formatstr);
  	vsprintf(msg,formatstr,args);
  	va_end(args);

  	// print verbose error message
  	printf("\n%s\n",msg);
		    
  	// print status (hex)
 	printf("GPIB status:\nibsta= %d (0x%x) ",ibsta,ibsta);

  	// print status (bin)
  	printf("<%1d%1d%1d%1d%1d%1d%1d%1d%1d%1d%1d%1d%1d%1d%1d%1d>\n<",
	 (ibsta&ERR  ?1:0),
	 (ibsta&TIMO ?1:0),
	 (ibsta&END  ?1:0),
	 (ibsta&SRQI ?1:0),
	 (ibsta&RQS  ?1:0),
	 (ibsta&EVENT?1:0),
	 (ibsta&SPOLL?1:0),
	 (ibsta&CMPL ?1:0),
	 (ibsta&LOK  ?1:0),
	 (ibsta&REM  ?1:0),
	 (ibsta&CIC  ?1:0),
	 (ibsta&ATN  ?1:0),
	 (ibsta&TACS ?1:0),
	 (ibsta&LACS ?1:0),
	 (ibsta&DTAS ?1:0),
	 (ibsta&DCAS ?1:0));

  	// print status bits (verbose)
  	if ( ibsta & ERR )  printf(" ERR");
  	if ( ibsta & TIMO ) printf(" TIMO");
  	if ( ibsta & END )  printf(" END");
  	if ( ibsta & SRQI ) printf(" SRQI");
 	if ( ibsta & RQS )  printf(" RQS");
  	if ( ibsta & EVENT ) printf(" EVENT");
  	if ( ibsta & SPOLL ) printf(" SPOLL");
  	if ( ibsta & CMPL ) printf(" CMPL");
  	if ( ibsta & LOK )  printf(" LOK");
  	if ( ibsta & REM )  printf(" REM");
  	if ( ibsta & CIC )  printf(" CIC");
  	if ( ibsta & ATN )  printf(" ATM");
  	if ( ibsta & TACS ) printf(" TACS");
  	if ( ibsta & LACS ) printf(" LACS");
  	if ( ibsta & DTAS ) printf(" DATS");
  	if ( ibsta & DCAS ) printf(" DCTS");
  	printf(" >\n");

  	// print error codes (verbose)
  	printf("iberr= %d:", iberr);
  	if ( iberr == EDVR) printf(" EDVR <System Error>\n");
  	if ( iberr == ECIC) printf(" ECIC <Not Controller-In-Charge (CIC)>\n");
  	if ( iberr == ENOL) printf(" ENOL <No Listener>\n");
  	if ( iberr == EADR) printf(" EADR <Address Error>\n");
  	if ( iberr == EARG) printf(" EARG <Invalid Argument>\n");
  	if ( iberr == ESAC) printf(" ESAC <No System Controller>\n");
  	if ( iberr == EABO) printf(" EABO <I/O Operation Aborted>\n");
  	if ( iberr == ENEB) printf(" ENEB <No GPIB Board>\n");
  	if ( iberr == EDMA) printf(" EDMA <DMA Hardware Error>\n");
  	//if ( iberr == EBTO) printf(" EBTO <DMA uP Bus Timeout>\n");
  	if ( iberr == EOIP) printf(" EOIP <Async I/O In Prg>\n");
 	if ( iberr == ECAP) printf(" ECAP <No Capability>\n");
  	if ( iberr == EFSO) printf(" EFSO <File System Error>\n");
  	//if ( iberr == EOWN) printf(" EOWN <Board Not Shareable>\n");
 	if ( iberr == EBUS) printf(" EBUS <Bus Error>\n");
  	if ( iberr == ESTB) printf(" ESTB <Serial Poll Byte(s) Lost>\n");
  	if ( iberr == ESRQ) printf(" ESRQ <SRQ Stuck On>\n");
  	//if ( iberr == ECFG) printf(" ECFG <Config File Not Found>\n");
  	//if ( iberr == EPAR) printf(" EPAR <Parse Error in Config>\n");
  	if ( iberr == ETAB) printf(" ETAB <Table Overflow>\n");
  	//if ( iberr == ENSD) printf(" ENSD <Device Not Found>\n");
  	//if ( iberr == ENWE) printf(" ENWE <Network Error>\n");
  	//if ( iberr == ENTF) printf(" ENTF <Nethandle-Table Full>\n");
  	//if ( iberr == EMEM) printf(" EMEM <Memory Allocation Error>\n");

  	// print bytecount
  	printf("ibcnt= %d\n", ibcnt );
  	printf("\n");


  	/*ibonl(0);*/
}

void  gpiberr ( char *errmsgstr ) {
  gpib_error(errmsgstr);
}
